#include "driver/i2s.h"
#include "freertos/queue.h"
#include "Tables/SinTable.h"
#include "Tables/CosTable.h"

#define SAMPLERATE 48000
#define TEMP 2000

int32_t rez_angle = 0; // угол поворота вала
uint32_t i = 0; // счетчик темпа ввода

// Переменные I2S
const i2s_port_t I2S_PORT_IN = I2S_NUM_0; // порт для ввода сигнала
int64_t ADC_Read = 0; // читаем выборку 32*2=64 бита из АЦП по I2S 
int32_t ADC_Read32 = 0; // входная синусоида

const i2s_port_t I2S_PORT_OUT = I2S_NUM_1; // порт для вывода сигнала
int64_t DAC_Write = 0; // пишем 32*2=64 бита в ЦАП по I2S 
int32_t DAC_Write_sin = 0; // синус с резольвера
int32_t DAC_Write_cos = 0; // косинус с резольвера

void setup(){
   // Serial port for debugging purposes
     Serial.begin(115200);
    
  // Настройка I2S
    Serial.println("Configuring I2S...");
    esp_err_t err;
  
  const i2s_config_t i2s_config_in = {
      .mode = i2s_mode_t(I2S_MODE_SLAVE | I2S_MODE_RX),
      .sample_rate = SAMPLERATE,                       
      .bits_per_sample = I2S_BITS_PER_SAMPLE_32BIT,
      .channel_format = I2S_CHANNEL_FMT_RIGHT_LEFT,  
      .communication_format = i2s_comm_format_t(I2S_COMM_FORMAT_I2S_MSB | I2S_COMM_FORMAT_I2S_MSB),
      .intr_alloc_flags = ESP_INTR_FLAG_LEVEL1,  
      .dma_buf_count = 8,    
      .dma_buf_len = 8,
      .use_apll = false,
    };
   
    i2s_pin_config_t pin_config_in = {
      .bck_io_num = 18,  // PCM1808 BCLK
      .ws_io_num = 19,   // PCM1808 LR
      .data_out_num = I2S_PIN_NO_CHANGE, // not used
      .data_in_num = 21  // PCM1808 SD
    };
    
    const i2s_config_t i2s_config_out = {
      .mode = i2s_mode_t(I2S_MODE_MASTER | I2S_MODE_TX),
      .sample_rate = SAMPLERATE,                        
      .bits_per_sample = I2S_BITS_PER_SAMPLE_32BIT, 
      .channel_format = I2S_CHANNEL_FMT_RIGHT_LEFT, 
      .communication_format = i2s_comm_format_t(I2S_COMM_FORMAT_I2S_MSB | I2S_COMM_FORMAT_I2S_MSB),
      .intr_alloc_flags = ESP_INTR_FLAG_LEVEL1,     
      .dma_buf_count = 8,                           
      .dma_buf_len = 8,
      .use_apll = false,
      .tx_desc_auto_clear= true
    };
  
  i2s_pin_config_t pin_config_out = {
      .bck_io_num = 5,  // BCLK PCM5102
      .ws_io_num = 17,   // LR PCM5102
      .data_out_num = 16,// SD PCM5102
      .data_in_num = I2S_PIN_NO_CHANGE, // not used
    }; 

  err = i2s_driver_install(I2S_PORT_IN, &i2s_config_in, 0, NULL);
  if (err != ESP_OK) {
    Serial.printf("Failed installing INPUT driver: %d\n", err);
    while (true);
    }
  
  err = i2s_driver_install(I2S_PORT_OUT, &i2s_config_out, 0, NULL);
  if (err != ESP_OK) {
    Serial.printf("Failed installing OUTPUT driver: %d\n", err);
    while (true);
    }

  err = i2s_set_pin(I2S_PORT_IN, &pin_config_in);
  if (err != ESP_OK) {
    Serial.printf("Failed setting DIN pin: %d\n", err);
    while (true);
    }
  
  err = i2s_set_pin(I2S_PORT_OUT, &pin_config_out);
  if (err != ESP_OK) {
    Serial.printf("Failed setting  DOUT pin: %d\n", err);
    while (true);
    }

    i2s_driver_install(I2S_PORT_IN, &i2s_config_in, 0, NULL);        // инсталляция и запуск драйвера входного I2S
    i2s_set_pin(I2S_PORT_IN, &pin_config_in);                        // запись портов для входного I2S 
    i2s_driver_install(I2S_PORT_OUT, &i2s_config_out, 0, NULL);      // инсталляция и запуск драйвера выходного I2S
    i2s_set_pin(I2S_PORT_OUT, &pin_config_out);                      // запись портов для выходного I2S   

    Serial.println("I2S driver installed and running.");

    // Вывод на порт 0 MCLK I2S0 сигнала синхронизации MCLK для устройств I2S 
    REG_WRITE(PIN_CTRL, 0b111111110000); 
    PIN_FUNC_SELECT(PERIPHS_IO_MUX_GPIO0_U, FUNC_GPIO0_CLK_OUT1);

    // Кнопки управления углом
    pinMode(33,INPUT_PULLUP); // кнопка "меньше"
    pinMode(32,INPUT_PULLUP); // кнопка "больше"
}  

void loop() {
      
   // Читаем PCM1808
    size_t bytesRead = 0;
    if (i2s_read(I2S_PORT_IN, &ADC_Read, 8, &bytesRead, portMAX_DELAY) == ESP_OK) {
    ADC_Read32 = ADC_Read & 0x00000000FFFFFFFF; // вырезаем младшие 32 разряда
  };  

    // Управление углом с помощью кнопок "больше" и "меньше"
    i++;  
    if (i/TEMP == 1){
      i = 0;      
      if (digitalRead(33) == LOW) rez_angle--;
      if (rez_angle < 0) rez_angle = 0; 
      if (digitalRead(32) == LOW) rez_angle++;
      if (rez_angle > 360) rez_angle = 360; 
     }
   
    // Формируем выходные сигналы симулятора
    DAC_Write_sin = ADC_Read32 * 2 * SinTable[rez_angle];
    DAC_Write_cos = ADC_Read32 * 2 * CosTable[rez_angle];
    
    // Записываем PCM5102
    DAC_Write = (DAC_Write_sin / 2) + ((DAC_Write_cos * 0x100000000) / 2);
    size_t bytesWritten = 0;     
    i2s_write(I2S_PORT_OUT, &DAC_Write, 8, &bytesWritten, portMAX_DELAY);
}
